@@ -1,8 +1,8 @@
 /*
  *      editor.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -14,25 +14,33 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- * $Id: editor.h 5432 2010-11-23 17:36:29Z ntrel $
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 
 #ifndef GEANY_EDITOR_H
 #define GEANY_EDITOR_H 1
 
-#include "Scintilla.h"
-#include "ScintillaWidget.h"
+#include "tm_tag.h" /* for TMTag */
+
+#include "gtkcompat.h" /* Needed by ScintillaWidget.h */
+#include "Scintilla.h" /* Needed by ScintillaWidget.h */
+#include "ScintillaWidget.h" /* for ScintillaObject */
+
+#include <glib.h>
+
+
+G_BEGIN_DECLS
+
+/* Forward-declared to avoid including document.h since it includes this header */
+struct GeanyDocument;
 
 /** Default character set to define which characters should be treated as part of a word. */
 #define GEANY_WORDCHARS					"_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
 #define GEANY_MAX_WORD_LENGTH			192
 
-
 /** Whether to use tabs, spaces or both to indent. */
 typedef enum
 {
@@ -51,24 +59,6 @@ typedef enum
 }
 GeanyAutoIndent;
 
-typedef enum
-{
-	GEANY_VIRTUAL_SPACE_DISABLED = 0,
-	GEANY_VIRTUAL_SPACE_SELECTION = 1,
-	GEANY_VIRTUAL_SPACE_ALWAYS = 3
-}
-GeanyVirtualSpace;
-
-
-/* Auto-close brackets/quotes */
-enum {
-	GEANY_AC_PARENTHESIS	= 1,
-	GEANY_AC_CBRACKET		= 2,
-	GEANY_AC_SBRACKET		= 4,
-	GEANY_AC_SQUOTE			= 8,
-	GEANY_AC_DQUOTE			= 16
-};
-
 /** Geany indicator types, can be used with Editor indicator functions to highlight
  *  text in the document. */
 typedef enum
@@ -96,23 +86,21 @@ typedef struct GeanyIndentPrefs
 	gint			hard_tab_width;
 	GeanyAutoIndent	auto_indent_mode;
 	gboolean		detect_type;
+	gboolean		detect_width;
 }
 GeanyIndentPrefs;
 
-
 /** Default prefs when creating a new editor window.
  * Some of these can be overridden per document or per project. */
-/* See editor_get_prefs(). */
+/* @warning Use @c editor_get_prefs() instead to include project overrides. */
 typedef struct GeanyEditorPrefs
 {
 	GeanyIndentPrefs *indentation;	/* Default indentation prefs. Use editor_get_indent_prefs(). */
 	gboolean	show_white_space;
 	gboolean	show_indent_guide;
 	gboolean	show_line_endings;
-	/* 0 - line, 1 - background, 2 - disabled.
-	 * This setting may be overridden when a project is opened. Use @c editor_get_prefs(). */
+	/* 0 - line, 1 - background, 2 - disabled. */
 	gint		long_line_type;
-	/* This setting may be overridden when a project is opened. Use @c editor_get_prefs(). */
 	gint		long_line_column;
 	gchar		*long_line_color;
 	gboolean	show_markers_margin;		/* view menu */
@@ -144,18 +132,15 @@ typedef struct GeanyEditorPrefs
 	gboolean	completion_drops_rest_of_word;
 	gchar		*color_scheme;
 	gint 		show_virtual_space;
-	/* This setting may be overridden when a project is opened. Use @c editor_get_prefs(). */
 	gboolean	long_line_enabled;
+	gint		autocompletion_update_freq;
 }
 GeanyEditorPrefs;
 
-extern GeanyEditorPrefs editor_prefs;
-
-
 /** Editor-owned fields for each document. */
-struct GeanyEditor
+typedef struct GeanyEditor
 {
-	GeanyDocument	*document;		/**< The document associated with the editor. */
+	struct GeanyDocument	*document;		/**< The document associated with the editor. */
 	ScintillaObject	*sci;			/**< The Scintilla editor @c GtkWidget. */
 	gboolean		 line_wrapping;	/**< @c TRUE if line wrapping is enabled. */
 	gboolean		 auto_indent;	/**< @c TRUE if auto-indentation is enabled. */
@@ -164,8 +149,65 @@ struct GeanyEditor
 	GeanyIndentType	 indent_type;	/* Use editor_get_indent_prefs() instead. */
 	gboolean		 line_breaking;	/**< Whether to split long lines as you type. */
 	gint			 indent_width;
-};
+}
+GeanyEditor;
 
+typedef struct SCNotification SCNotification;
+
+
+const GeanyIndentPrefs *editor_get_indent_prefs(GeanyEditor *editor);
+
+ScintillaObject *editor_create_widget(GeanyEditor *editor);
+
+void editor_indicator_set_on_range(GeanyEditor *editor, gint indic, gint start, gint end);
+
+void editor_indicator_set_on_line(GeanyEditor *editor, gint indic, gint line);
+
+void editor_indicator_clear(GeanyEditor *editor, gint indic);
+
+void editor_set_indent_type(GeanyEditor *editor, GeanyIndentType type);
+
+gchar *editor_get_word_at_pos(GeanyEditor *editor, gint pos, const gchar *wordchars);
+
+const gchar *editor_get_eol_char_name(GeanyEditor *editor);
+
+gint editor_get_eol_char_len(GeanyEditor *editor);
+
+const gchar *editor_get_eol_char(GeanyEditor *editor);
+
+void editor_insert_text_block(GeanyEditor *editor, const gchar *text,
+	 						  gint insert_pos, gint cursor_index,
+	 						  gint newline_indent_size, gboolean replace_newlines);
+
+gint editor_get_eol_char_mode(GeanyEditor *editor);
+
+gboolean editor_goto_pos(GeanyEditor *editor, gint pos, gboolean mark);
+
+const gchar *editor_find_snippet(GeanyEditor *editor, const gchar *snippet_name);
+
+void editor_insert_snippet(GeanyEditor *editor, gint pos, const gchar *snippet);
+
+
+#ifdef GEANY_PRIVATE
+
+extern GeanyEditorPrefs editor_prefs;
+
+typedef enum
+{
+	GEANY_VIRTUAL_SPACE_DISABLED = 0,
+	GEANY_VIRTUAL_SPACE_SELECTION = 1,
+	GEANY_VIRTUAL_SPACE_ALWAYS = 3
+}
+GeanyVirtualSpace;
+
+/* Auto-close brackets/quotes */
+enum {
+	GEANY_AC_PARENTHESIS	= 1,
+	GEANY_AC_CBRACKET		= 2,
+	GEANY_AC_SBRACKET		= 4,
+	GEANY_AC_SQUOTE			= 8,
+	GEANY_AC_DQUOTE			= 16
+};
 
 typedef struct
 {
@@ -175,17 +217,13 @@ typedef struct
 
 extern EditorInfo editor_info;
 
-typedef struct SCNotification SCNotification;
-
 
 void editor_init(void);
 
-GeanyEditor *editor_create(GeanyDocument *doc);
+GeanyEditor *editor_create(struct GeanyDocument *doc);
 
 void editor_destroy(GeanyEditor *editor);
 
-ScintillaObject *editor_create_widget(GeanyEditor *editor);
-
 void editor_sci_notify_cb(GtkWidget *widget, gint scn, gpointer scnt, gpointer data);
 
 gboolean editor_start_auto_complete(GeanyEditor *editor, gint pos, gboolean force);
@@ -196,18 +234,15 @@ void editor_goto_next_snippet_cursor(GeanyEditor *editor);
 
 gboolean editor_complete_snippet(GeanyEditor *editor, gint pos);
 
-void editor_show_macro_list(GeanyEditor *editor);
-
 gboolean editor_show_calltip(GeanyEditor *editor, gint pos);
 
 void editor_do_comment_toggle(GeanyEditor *editor);
 
-void editor_do_comment(GeanyEditor *editor, gint line, gboolean allow_empty_lines, gboolean toggle);
+gint editor_do_comment(GeanyEditor *editor, gint line, gboolean allow_empty_lines, gboolean toggle,
+		gboolean single_comment);
 
 gint editor_do_uncomment(GeanyEditor *editor, gint line, gboolean toggle);
 
-gint editor_lexer_get_type_keyword_idx(gint lexer);
-
 void editor_insert_multiline_comment(GeanyEditor *editor);
 
 void editor_insert_alternative_whitespace(GeanyEditor *editor);
@@ -235,10 +270,10 @@ const GeanyEditorPrefs *editor_get_prefs(GeanyEditor *editor);
 
 /* General editing functions */
 
-void editor_find_current_word(GeanyEditor *editor, gint pos, gchar *word, size_t wordlen,
+void editor_find_current_word(GeanyEditor *editor, gint pos, gchar *word, gsize wordlen,
 	const gchar *wc);
 
-gchar *editor_get_word_at_pos(GeanyEditor *editor, gint pos, const gchar *wordchars);
+void editor_find_current_word_sciwc(GeanyEditor *editor, gint pos, gchar *word, gsize wordlen);
 
 gchar *editor_get_default_selection(GeanyEditor *editor, gboolean use_current_word, const gchar *wordchars);
 
@@ -254,48 +289,30 @@ void editor_select_indent_block(GeanyEditor *editor);
 
 void editor_set_font(GeanyEditor *editor, const gchar *font);
 
-void editor_indicator_set_on_line(GeanyEditor *editor, gint indic, gint line);
-
 void editor_indicator_clear_errors(GeanyEditor *editor);
 
-void editor_indicator_set_on_range(GeanyEditor *editor, gint indic, gint start, gint end);
-
-void editor_indicator_clear(GeanyEditor *editor, gint indic);
-
-gint editor_get_eol_char_mode(GeanyEditor *editor);
-
-const gchar *editor_get_eol_char_name(GeanyEditor *editor);
-
-gint editor_get_eol_char_len(GeanyEditor *editor);
-
-const gchar *editor_get_eol_char(GeanyEditor *editor);
-
 void editor_fold_all(GeanyEditor *editor);
 
 void editor_unfold_all(GeanyEditor *editor);
 
-void editor_replace_tabs(GeanyEditor *editor);
+void editor_replace_tabs(GeanyEditor *editor, gboolean ignore_selection);
 
-void editor_replace_spaces(GeanyEditor *editor);
+void editor_replace_spaces(GeanyEditor *editor, gboolean ignore_selection);
 
 void editor_strip_line_trailing_spaces(GeanyEditor *editor, gint line);
 
-void editor_strip_trailing_spaces(GeanyEditor *editor);
+void editor_strip_trailing_spaces(GeanyEditor *editor, gboolean ignore_selection);
 
 void editor_ensure_final_newline(GeanyEditor *editor);
 
 void editor_insert_color(GeanyEditor *editor, const gchar *colour);
 
-const GeanyIndentPrefs *editor_get_indent_prefs(GeanyEditor *editor);
-
-void editor_set_indent_type(GeanyEditor *editor, GeanyIndentType type);
+void editor_set_indent_width(GeanyEditor *editor, gint width);
 
 void editor_set_indent(GeanyEditor *editor, GeanyIndentType type, gint width);
 
 void editor_set_line_wrapping(GeanyEditor *editor, gboolean wrap);
 
-gboolean editor_goto_pos(GeanyEditor *editor, gint pos, gboolean mark);
-
 gboolean editor_goto_line(GeanyEditor *editor, gint line_no, gint offset);
 
 void editor_set_indentation_guides(GeanyEditor *editor);
@@ -304,14 +321,10 @@ void editor_apply_update_prefs(GeanyEditor *editor);
 
 gchar *editor_get_calltip_text(GeanyEditor *editor, const TMTag *tag);
 
-void editor_insert_text_block(GeanyEditor *editor, const gchar *text,
-	 						  gint insert_pos, gint cursor_index,
-	 						  gint newline_indent_size, gboolean replace_newlines);
-
 void editor_toggle_fold(GeanyEditor *editor, gint line, gint modifiers);
 
-const gchar *editor_find_snippet(GeanyEditor *editor, const gchar *snippet_name);
+#endif /* GEANY_PRIVATE */
 
-void editor_insert_snippet(GeanyEditor *editor, gint pos, const gchar *snippet);
+G_END_DECLS
 
-#endif
+#endif /* GEANY_EDITOR_H */
