@@ -1,8 +1,8 @@
 /*
  *      utils.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -14,11 +14,9 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: utils.h 5389 2010-11-09 18:17:57Z ntrel $
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /**
@@ -29,19 +27,31 @@
 #ifndef GEANY_UTILS_H
 #define GEANY_UTILS_H 1
 
-
+#include <string.h>
 #include <time.h>
 
+#include <glib.h>
+#include <gdk/gdk.h> /* for GdkColor */
 
-/** Returns TRUE if @a ptr points to a non-zero value. */
-#define NZV(ptr) \
-	(G_LIKELY((ptr)) && G_LIKELY((ptr)[0]))
+G_BEGIN_DECLS
 
-/**
- *  Frees @a ptr (if not @c NULL), then assigns @a result to it.
- *  @a result can be an expression using the 'old' value of @a ptr.
- *  It prevents a memory leak compared with: @code ptr = func(ptr); @endcode
+/** Returns @c TRUE if @a ptr is @c NULL or @c *ptr is @c FALSE. */
+#define EMPTY(ptr) \
+	(!(ptr) || !*(ptr))
+
+/** @deprecated 2013/08 - use @c !EMPTY() instead. */
+#ifndef GEANY_DISABLE_DEPRECATED
+#define NZV(ptr) (!EMPTY(ptr))
+#endif
+
+/** Assigns @a result to @a ptr, then frees the old value.
+ * @a result can be an expression using the 'old' value of @a ptr.
+ * E.g. @code SETPTR(str, g_strndup(str, 5)); @endcode
  **/
+#define SETPTR(ptr, result) \
+	do setptr(ptr, result) while (0)
+
+/** @deprecated 2011/11/15 - use SETPTR() instead. */
 #define setptr(ptr, result) \
 	{\
 		gpointer setptr_tmp = ptr;\
@@ -64,6 +74,13 @@
 		utils_get_setting_##type(home, group, key, default_val) :\
 		utils_get_setting_##type(sys, group, key, default_val)
 
+/* ignore the case of filenames and paths under WIN32, causes errors if not */
+#ifdef G_OS_WIN32
+#define utils_filenamecmp(a, b)	utils_str_casecmp((a), (b))
+#else
+#define utils_filenamecmp(a, b)	strcmp((a), (b))
+#endif
+
 
 /** Iterates all the items in @a array using pointers.
  * @param item pointer to an item in @a array.
@@ -84,8 +101,8 @@
  * @param idx @c guint index into @a ptr_array.
  * @param ptr_array @c GPtrArray to traverse. */
 #define foreach_ptr_array(item, idx, ptr_array) \
-	for (idx = 0, item = g_ptr_array_index(ptr_array, 0); \
-		idx < ptr_array->len; ++idx, item = g_ptr_array_index(ptr_array, idx))
+	for (idx = 0, item = ((ptr_array)->len > 0 ? g_ptr_array_index((ptr_array), 0) : NULL); \
+		idx < (ptr_array)->len; ++idx, item = g_ptr_array_index((ptr_array), idx))
 
 /** Iterates all the nodes in @a list.
  * @param node should be a (@c GList*).
@@ -99,6 +116,14 @@
 #define foreach_slist(node, list) \
 	foreach_list(node, list)
 
+/* Iterates all the nodes in @a list. Safe against removal during iteration
+ * @param node should be a (@c GList*).
+ * @param list @c GList to traverse. */
+#define foreach_list_safe(node, list) \
+	for (GList *_node = (list), *_next = (list) ? (list)->next : NULL; \
+	     (node = _node) != NULL; \
+	     _node = _next, _next = _next ? _next->next : NULL)
+
 /** Iterates through each unsorted filename in a @c GDir.
  * @param filename (@c const @c gchar*) locale-encoded filename, without path. Do not modify or free.
  * @param dir @c GDir created with @c g_dir_open(). Call @c g_dir_close() afterwards.
@@ -130,18 +155,77 @@
 	for (i = 0; i < size; i++)
 
 
+gboolean utils_str_equal(const gchar *a, const gchar *b);
+
+guint utils_string_replace_all(GString *haystack, const gchar *needle, const gchar *replace);
+
+GSList *utils_get_file_list(const gchar *path, guint *length, GError **error);
+
+GSList *utils_get_file_list_full(const gchar *path, gboolean full_path, gboolean sort, GError **error);
+
+gint utils_write_file(const gchar *filename, const gchar *text);
+
+gchar *utils_get_locale_from_utf8(const gchar *utf8_text);
+
+gchar *utils_get_utf8_from_locale(const gchar *locale_text);
+
+gchar *utils_remove_ext_from_filename(const gchar *filename);
+
+gint utils_mkdir(const gchar *path, gboolean create_parent_dirs);
+
+gboolean utils_get_setting_boolean(GKeyFile *config, const gchar *section, const gchar *key, const gboolean default_value);
+
+gint utils_get_setting_integer(GKeyFile *config, const gchar *section, const gchar *key, const gint default_value);
+
+gchar *utils_get_setting_string(GKeyFile *config, const gchar *section, const gchar *key, const gchar *default_value);
+
+gboolean utils_spawn_sync(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags flags,
+						  GSpawnChildSetupFunc child_setup, gpointer user_data, gchar **std_out,
+						  gchar **std_err, gint *exit_status, GError **error);
+
+gboolean utils_spawn_async(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags flags,
+						   GSpawnChildSetupFunc child_setup, gpointer user_data, GPid *child_pid,
+						   GError **error);
+
+gint utils_str_casecmp(const gchar *s1, const gchar *s2);
+
+gchar *utils_get_date_time(const gchar *format, time_t *time_to_use);
+
 void utils_open_browser(const gchar *uri);
 
-gint utils_get_line_endings(const gchar* buffer, glong size);
+guint utils_string_replace_first(GString *haystack, const gchar *needle, const gchar *replace);
+
+gchar *utils_str_middle_truncate(const gchar *string, guint truncate_length);
+
+gchar *utils_str_remove_chars(gchar *string, const gchar *chars);
+
+gchar **utils_copy_environment(const gchar **exclude_vars, const gchar *first_varname, ...) G_GNUC_NULL_TERMINATED;
+
+gchar *utils_find_open_xml_tag(const gchar sel[], gint size);
+
+const gchar *utils_find_open_xml_tag_pos(const gchar sel[], gint size);
+
+
+#ifdef GEANY_PRIVATE
+
+typedef enum
+{
+	RESOURCE_DIR_DATA,
+	RESOURCE_DIR_ICON,
+	RESOURCE_DIR_DOC,
+	RESOURCE_DIR_LOCALE,
+	RESOURCE_DIR_PLUGIN,
+
+	RESOURCE_DIR_COUNT
+} GeanyResourceDirType;
+
+
+gint utils_get_line_endings(const gchar* buffer, gsize size);
 
 gboolean utils_isbrace(gchar c, gboolean include_angles);
 
 gboolean utils_is_opening_brace(gchar c, gboolean include_angles);
 
-gint utils_write_file(const gchar *filename, const gchar *text);
-
-gchar *utils_find_open_xml_tag(const gchar sel[], gint size);
-
 gboolean utils_is_short_html_tag(const gchar *tag_name);
 
 void utils_ensure_same_eol_characters(GString *string, gint target_eol_mode);
@@ -150,6 +234,8 @@ const gchar *utils_get_eol_char(gint eol_mode);
 
 const gchar *utils_get_eol_name(gint eol_mode);
 
+const gchar *utils_get_eol_short_name(gint eol_mode);
+
 gboolean utils_atob(const gchar *str);
 
 void utils_tidy_path(gchar *filename);
@@ -160,36 +246,23 @@ const gchar *utils_path_skip_root(const gchar *path);
 
 gdouble utils_scale_round(gdouble val, gdouble factor);
 
-gboolean utils_str_equal(const gchar *a, const gchar *b);
-
-gchar *utils_remove_ext_from_filename(const gchar *filename);
-
 gchar utils_brace_opposite(gchar ch);
 
-gchar *utils_get_hostname(void);
+gint utils_string_find(GString *haystack, gint start, gint end, const gchar *needle);
 
-guint utils_string_replace_all(GString *haystack, const gchar *needle, const gchar *replace);
+gint utils_string_replace(GString *str, gint pos, gint len, const gchar *replace);
 
-guint utils_string_replace_first(GString *haystack, const gchar *needle, const gchar *replace);
+guint utils_string_regex_replace_all(GString *haystack, GRegex *regex,
+		guint match_num, const gchar *replace, gboolean literal);
 
 void utils_str_replace_all(gchar **haystack, const gchar *needle, const gchar *replacement);
 
 gint utils_strpos(const gchar* haystack, const gchar *needle);
 
-gchar *utils_get_date_time(const gchar *format, time_t *time_to_use);
-
 gchar *utils_get_initials(const gchar *name);
 
-gboolean utils_get_setting_boolean(GKeyFile *config, const gchar *section, const gchar *key, const gboolean default_value);
-
-gint utils_get_setting_integer(GKeyFile *config, const gchar *section, const gchar *key, const gint default_value);
-
-gchar *utils_get_setting_string(GKeyFile *config, const gchar *section, const gchar *key, const gchar *default_value);
-
 gchar *utils_get_hex_from_color(GdkColor *color);
 
-guint utils_invert_color(guint color);
-
 const gchar *utils_get_default_dir_utf8(void);
 
 gchar *utils_get_current_file_dir_utf8(void);
@@ -199,7 +272,11 @@ void utils_beep(void);
 gchar *utils_make_human_readable_str(guint64 size, gulong block_size,
 									 gulong display_unit);
 
-gint utils_strtod(const gchar *source, gchar **end, gboolean with_route);
+gboolean utils_parse_color(const gchar *spec, GdkColor *color);
+
+gint utils_color_to_bgr(const GdkColor *color);
+
+gint utils_parse_color_to_bgr(const gchar *spec);
 
 gchar *utils_get_current_time_string(void);
 
@@ -212,53 +289,39 @@ gboolean utils_str_replace_escape(gchar *string, gboolean keep_backslash);
 
 gboolean utils_wrap_string(gchar *string, gint wrapstart);
 
-gchar *utils_get_locale_from_utf8(const gchar *utf8_text);
-
-gchar *utils_get_utf8_from_locale(const gchar *locale_text);
-
 void utils_free_pointers(gsize arg_count, ...) G_GNUC_NULL_TERMINATED;
 
 gchar **utils_strv_new(const gchar *first, ...) G_GNUC_NULL_TERMINATED;
 
 gchar **utils_strv_join(gchar **first, gchar **second) G_GNUC_WARN_UNUSED_RESULT;
 
-gint utils_mkdir(const gchar *path, gboolean create_parent_dirs);
-
-GSList *utils_get_file_list(const gchar *path, guint *length, GError **error);
-
-GSList *utils_get_file_list_full(const gchar *path, gboolean full_path, gboolean sort, GError **error);
-
 GSList *utils_get_config_files(const gchar *subdir);
 
 gchar *utils_get_help_url(const gchar *suffix);
 
 gboolean utils_str_has_upper(const gchar *str);
 
-gint utils_is_file_writeable(const gchar *locale_filename);
+gint utils_is_file_writable(const gchar *locale_filename);
 
 
-gboolean utils_spawn_sync(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags flags,
-						  GSpawnChildSetupFunc child_setup, gpointer user_data, gchar **std_out,
-						  gchar **std_err, gint *exit_status, GError **error);
-
-gboolean utils_spawn_async(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags flags,
-						   GSpawnChildSetupFunc child_setup, gpointer user_data, GPid *child_pid,
-						   GError **error);
-
-gint utils_str_casecmp(const gchar *s1, const gchar *s2);
-
-const gchar *utils_build_path(const gchar *first, ...) G_GNUC_NULL_TERMINATED;
-
 gchar *utils_get_path_from_uri(const gchar *uri);
 
 gboolean utils_is_uri(const gchar *uri);
 
 gboolean utils_is_remote_path(const gchar *path);
 
-gchar *utils_str_middle_truncate(const gchar *string, guint truncate_length);
+GDate *utils_parse_date(const gchar *input);
 
-gchar *utils_str_remove_chars(gchar *string, const gchar *chars);
+gchar *utils_parse_and_format_build_date(const gchar *input);
 
-gchar **utils_copy_environment(const gchar **exclude_vars, const gchar *first_varname, ...) G_GNUC_NULL_TERMINATED;
+gchar *utils_get_user_config_dir(void);
 
-#endif
+const gchar *utils_resource_dir(GeanyResourceDirType type);
+
+void utils_start_new_geany_instance(const gchar *doc_path);
+
+#endif /* GEANY_PRIVATE */
+
+G_END_DECLS
+
+#endif /* GEANY_UTILS_H */
